/**
 * Any copyright is dedicated to the Public Domain.
 * http://creativecommons.org/publicdomain/zero/1.0/
 */

/* import-globals-from databaseShadowing-shared.js */
loadSubscript("databaseShadowing-shared.js");

add_task(async function testSteps() {
  enableNextGenLocalStorage();

  storeData();

  verifyData([]);

  // Wait for all database connections to close.
  let request = reset();
  await requestFinished(request);

  exportShadowDatabase("shadowdb.sqlite");

  // The shadow database is now prepared for test_databaseShadowing2.js
});
